-- Import the necessary modules for AndroLua+
require "import"
import "android.widget.*"
import "android.view.*"
import "android.app.*"
import "android.content.*"
import "android.net.*"
import "android.util.*"
import "android.webkit.*"
import "java.lang.*"
import "com.androlua.Http"

function check_permissions()
    -- Check for both read and write permissions
    local read_permission = activity.checkSelfPermission("android.permission.READ_EXTERNAL_STORAGE")
    local write_permission = activity.checkSelfPermission("android.permission.WRITE_EXTERNAL_STORAGE")

    -- If permissions are not granted, request them
    if read_permission == -1 then  -- Permission not granted
        activity.requestPermissions({ "android.permission.READ_EXTERNAL_STORAGE" }, 1)
    end

    if write_permission == -1 then  -- Permission not granted
        activity.requestPermissions({ "android.permission.WRITE_EXTERNAL_STORAGE" }, 1)
    end
end

check_permissions()

-- Import Android libraries with proper Lua style
local ArrayAdapter = luajava.bindClass("android.widget.ArrayAdapter")
local AdapterView = luajava.bindClass("android.widget.AdapterView")
local LinearLayout = luajava.bindClass("android.widget.LinearLayout")
local EditText = luajava.bindClass("android.widget.EditText")
local Toast = luajava.bindClass("android.widget.Toast")
local Spinner = luajava.bindClass("android.widget.Spinner")
local ListView = luajava.bindClass("android.widget.ListView")
local Button = luajava.bindClass("android.widget.Button")
local WebView = luajava.bindClass("android.webkit.WebView")
local WebSettings = luajava.bindClass("android.webkit.WebSettings")
local AlertDialog = luajava.bindClass("android.app.AlertDialog")
local Log = luajava.bindClass("android.util.Log")
local Intent = luajava.bindClass("android.content.Intent")
local Uri = luajava.bindClass("android.net.Uri")
local TextWatcher = luajava.bindClass("android.text.TextWatcher")
local String = luajava.bindClass("java.lang.String")

-- Correct way to require a Lua module
local rss_feeds = require("rss")

local selected_language = "English"
local selected_category = "General"
local news_data = {}

-- Function to clean RSS URL
function clean_url(url)
    if not url then return "" end
    return url:gsub("<!%[CDATA%[", ""):gsub("%]%]>", ""):gsub("^%s*(.-)%s*$", "%1")
end

-- Function to parse RSS feed
function parse_rss(xml)
    local news_data = {}
    for item in xml:gmatch("<item>(.-)</item>") do
        local title = item:match("<title><!%[CDATA%[(.-)%]%]></title>") or
                       item:match("<title>(.-)</title>") or "No Title"
        local link = item:match("<link>(.-)</link>") or ""
        table.insert(news_data, { title = title, link = clean_url(link) })
    end
    return news_data
end

-- Helper function to dump tables for saving (needed for save_custom_feeds)
function dump(o)
    if type(o) == 'table' then
        local s = '{'
        for k,v in pairs(o) do
            if type(k) ~= 'number' then k = '"'..k..'"' end
            s = s .. '['..k..'] = ' .. dump(v) .. ','
        end
        return s .. '}'
    else
        return tostring(o)
    end
end

-- Load custom RSS feeds
local custom_feed_file = activity.getLuaDir() .. "/custom_feeds.lua"
function load_custom_feeds()
    local file = io.open(custom_feed_file, "r")
    if file then
        local content = file:read("*a")
        file:close()
        local chunk, err = loadstring(content)
        if chunk then
            return chunk() or {}
        else
            Log.e("RSS", "Error loading custom feeds: " .. tostring(err))
        end
    end
    return {}
end

-- Save custom RSS feeds
function save_custom_feeds(feeds)
    local file = io.open(custom_feed_file, "w")
    if file then
        file:write("return " .. dump(feeds))
        file:close()
    else
        Log.e("RSS", "Error saving custom feeds")
    end
end

-- Now calling load_custom_feeds() is safe
local custom_feeds = load_custom_feeds()

-- Load news feeds
function load_news()
    local custom_feeds = load_custom_feeds()
    local urls = {}
    Log.d("RSS", "Loading news for category: " .. selected_category)
    Log.d("RSS", "Selected language: " .. selected_language)
    if rss_feeds[selected_category] and rss_feeds[selected_category][selected_language] then
        for _, url in ipairs(rss_feeds[selected_category][selected_language]) do
            Log.d("RSS", "Adding URL from rss_feeds: " .. url)
            table.insert(urls, url)
        end
    else
        Log.e("RSS", "No feeds found in rss_feeds for selected category/language")
    end
    if custom_feeds[selected_category] and custom_feeds[selected_category][selected_language] then
        for _, url in ipairs(custom_feeds[selected_category][selected_language]) do
            Log.d("RSS", "Adding URL from custom_feeds: " .. url)
            table.insert(urls, url)
        end
    else
        Log.e("RSS", "No feeds found in custom_feeds for selected category/language")
    end
    if #urls == 0 then
        Log.e("RSS", "No URLs available to fetch news!")
        adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, String({"No news available"}))
        news_list.setAdapter(adapter)
        return
    end
    
    fetch_rss(urls, function(news)
        if #news == 0 then
            Log.e("RSS", "No news fetched from the URLs!")
        else
            Log.d("RSS", "News fetched: " .. #news)
        end
        news_data = news -- Store the fetched news data
        local titles = {}
        for _, item in ipairs(news) do
            table.insert(titles, item.title)
        end
        adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, String(titles))
        news_list.setAdapter(adapter)
    end)
end

-- Function to fetch RSS feeds
function fetch_rss(urls, callback)
    if not urls or #urls == 0 then 
        Log.e("RSS", "No URLs found for fetching news")
        callback({})
        return
    end
    local combined_news, count = {}, 0
    Toast.makeText(activity, "Loading News...", Toast.LENGTH_SHORT).show()
    for _, url in ipairs(urls) do
        Log.d("RSS", "Fetching: " .. url) -- Log each URL being fetched
        Http.get(url, function(status, body)
            count = count + 1
            if status == 200 then
                Log.d("RSS", "Successfully fetched: " .. url)
                Log.d("RSS", "Response body (first 500 chars): " .. body:sub(1, 500)) -- Print first 500 chars
                local news_data = parse_rss(body)
                for _, news in ipairs(news_data) do 
                    table.insert(combined_news, news) 
                end
            else 
                Log.e("RSS", "Failed to fetch: " .. url .. " | Status: " .. tostring(status)) 
            end
            if count == #urls then 
                callback(combined_news) 
            end
        end)
    end
end

-- Open news link in WebView
function open_webview(url)
    if url == "" or url == nil then
        Toast.makeText(activity, "No valid URL", Toast.LENGTH_SHORT).show()
        return
    end
    local intent = Intent(Intent.ACTION_VIEW, Uri.parse(url))
    activity.startActivity(intent)
end

layout = {
    LinearLayout, orientation = "vertical", layout_width = "fill", layout_height = "fill",
    { Spinner, id = "category_spinner", layout_width = "match", layout_height = "wrap" },
    { Spinner, id = "language_spinner", layout_width = "match", layout_height = "wrap" },
    { ListView, id = "news_list", layout_width = "fill", layout_height = "0dp", layout_weight = "1" },
    { Button, text = "Add Custom Feed", id = "add_feed_button", layout_width = "match", layout_height = "wrap" },
    { Button, text = "View Bookmarks", id = "bookmark_button", layout_width = "match", layout_height = "wrap" },
    { Button, text = "Read Offline News", id = "offline_news_button", layout_width = "match", layout_height = "wrap" },
    { Button, text = "About", id = "about_button", layout_width = "match", layout_height = "wrap" }
}

activity.setContentView(loadlayout(layout))

categories = {} 
for k, _ in pairs(rss_feeds) do 
    table.insert(categories, k) 
end

category_spinner.setAdapter(ArrayAdapter(activity, android.R.layout.simple_spinner_dropdown_item, String(categories)))
languages = { "English", "Hindi", "Nepali" }
language_spinner.setAdapter(ArrayAdapter(activity, android.R.layout.simple_spinner_dropdown_item, String(languages)))

load_news()

category_spinner.onItemSelected = function(_, _, position, _) 
    selected_category = categories[position + 1] 
    load_news() 
end

language_spinner.onItemSelected = function(_, _, position, _) 
    selected_language = languages[position + 1] 
    load_news() 
end

news_list.onItemClick = function(_, _, position, _) 
    open_webview(news_data[position + 1].link) 
end

local custom_feed_file = activity.getLuaDir() .. "/custom_feeds.lua"

function add_custom_feed()
    local url_input = EditText(activity)
    url_input.setHint("Enter RSS Feed URL")
    
    local category_spinner = Spinner(activity)
    local category_adapter = ArrayAdapter(activity, android.R.layout.simple_spinner_dropdown_item, String(categories))
    category_spinner.setAdapter(category_adapter)
    
    local new_category_input = EditText(activity)
    new_category_input.setHint("Or enter new category")
    
    local language_spinner = Spinner(activity)
    language_spinner.setAdapter(ArrayAdapter(activity, android.R.layout.simple_spinner_dropdown_item, String(languages)))
    
    local layout = LinearLayout(activity)
    layout.setOrientation(LinearLayout.VERTICAL)
    layout.addView(url_input)
    layout.addView(category_spinner)
    layout.addView(new_category_input)
    layout.addView(language_spinner)
    
    AlertDialog.Builder(activity)
        .setTitle("Add Custom Feed")
        .setView(layout)
        .setPositiveButton("Add", { onClick = function()
            local url = url_input.getText().toString()
            local selected_category = categories[category_spinner.getSelectedItemPosition() + 1]
            local new_category = new_category_input.getText().toString():match("^%s*(.-)%s*$") -- Trim spaces
            local language = languages[language_spinner.getSelectedItemPosition() + 1]
            
            if url ~= "" then
                -- Use new category if provided
                local category = (new_category ~= "" and new_category) or selected_category
                
                -- Add the category dynamically
                if new_category ~= "" and not rss_feeds[new_category] then
                    rss_feeds[new_category] = {}
                    table.insert(categories, new_category) -- Update UI category list
                    category_adapter.notifyDataSetChanged()
                end
                
                -- Ensure language exists
                if not rss_feeds[category][language] then
                    rss_feeds[category][language] = {}
                end
                
                table.insert(rss_feeds[category][language], url)
                
                -- Ensure custom_feeds is updated
                if not custom_feeds[category] then custom_feeds[category] = {} end
                if not custom_feeds[category][language] then custom_feeds[category][language] = {} end
                table.insert(custom_feeds[category][language], url)
                save_custom_feeds(custom_feeds) -- Save updated feeds
                
                Toast.makeText(activity, "Custom Feed Added!", Toast.LENGTH_SHORT).show()
                load_news()
            else
                Toast.makeText(activity, "Enter a valid URL!", Toast.LENGTH_SHORT).show()
            end
        end })
        .setNegativeButton("Cancel", nil)
        .show()
end

add_feed_button.onClick = add_custom_feed

local bookmarks_file = activity.getLuaDir() .. "/bookmarks.lua"

-- Function to serialize a table into a string
function dump(tbl)
    local result = "return {"
    for _, v in ipairs(tbl) do
        result = result .. string.format("{title=%q, link=%q},", v.title, v.link)
    end
    result = result .. "}"
    return result
end

-- Function to load bookmarks from a file
function load_bookmarks()
    local file = io.open(bookmarks_file, "r")
    if file then
        local content = file:read("*a")
        file:close()
        local chunk, err = loadstring(content)
        if chunk then
            return chunk() or {}
        else
            Log.e("Bookmarks", "Error loading bookmarks: " .. err)
        end
    end
    return {}
end

-- Function to save bookmarks to a file
function save_bookmarks(bookmarks)
    local file = io.open(bookmarks_file, "w")
    if file then
        file:write(dump(bookmarks))
        file:close()
    end
end

local bookmarks = load_bookmarks()

-- Function to add a news article to bookmarks
function add_to_bookmarks(news_item)
    if not news_item or not news_item.link or news_item.link == "" then
        Toast.makeText(activity, "Cannot bookmark this article", Toast.LENGTH_SHORT).show()
        return
    end

    -- Load existing bookmarks
    local bookmarks = load_bookmarks()

    -- Avoid duplicate bookmarks
    for _, b in ipairs(bookmarks) do
        if b.link == news_item.link then
            Toast.makeText(activity, "Already Bookmarked", Toast.LENGTH_SHORT).show()
            return
        end
    end

    table.insert(bookmarks, news_item)
    save_bookmarks(bookmarks)
    Toast.makeText(activity, "Added to Bookmarks", Toast.LENGTH_SHORT).show()
end

function show_bookmarks()
    local bookmarks = load_bookmarks()

    if #bookmarks == 0 then
        Toast.makeText(activity, "No Bookmarks Found", Toast.LENGTH_SHORT).show()
        return
    end

    local bookmark_titles = {}
    for _, b in ipairs(bookmarks) do
        table.insert(bookmark_titles, b.title)
    end

    local layout = LinearLayout(activity)
    layout.setOrientation(LinearLayout.VERTICAL)

    local list_view = ListView(activity)
    local adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, String(bookmark_titles))
    list_view.setAdapter(adapter)

    layout.addView(list_view)

    AlertDialog.Builder(activity)
        .setTitle("Bookmarked Articles")
        .setView(layout)
        .setNegativeButton("Close", nil)
        .show()

    -- जब बुकमार्क क्लिक गरिन्छ, लिङ्क खोल्ने
    list_view.setOnItemClickListener(AdapterView.OnItemClickListener {
        onItemClick = function(_, _, position, _)
            if bookmarks[position + 1] then
                open_webview(bookmarks[position + 1].link)
            end
        end
    })

    -- Long press गर्दा Delete र Share देखाउने
    list_view.setOnItemLongClickListener(AdapterView.OnItemLongClickListener {
        onItemLongClick = function(_, _, position, _)
            local selected_bookmark = bookmarks[position + 1]

            AlertDialog.Builder(activity)
                .setTitle("Options")
                .setItems({"Delete", "Share"}, { onClick = function(_, which)
                    if which == 0 then -- Delete
                        table.remove(bookmarks, position + 1)
                        save_bookmarks(bookmarks)
                        Toast.makeText(activity, "Bookmark Deleted", Toast.LENGTH_SHORT).show()
                        show_bookmarks() -- UI update
                    elseif which == 1 then -- Share
                        local intent = Intent(Intent.ACTION_SEND)
                        intent.setType("text/plain")
                        intent.putExtra(Intent.EXTRA_SUBJECT, selected_bookmark.title)
                        intent.putExtra(Intent.EXTRA_TEXT, selected_bookmark.link)
                        activity.startActivity(Intent.createChooser(intent, "Share via"))
                    end
                end})
                .setNegativeButton("Cancel", nil)
                .show()

            return true -- Long press handle गरियो
        end
    })
end

bookmark_button.onClick = show_bookmarks

function show_about()
    local about_message = [[
A simple RSS reader app for Android.
Created with AndroLua+

Version 1.0

For support or inquiries, contact us via email: chaudharisushil364@gmail.com

How to use:
1. **Add Custom RSS Feed**: 
   - Go to the ADD CUSTOM FEED section of the app and input your preferred RSS feed URLs.
   - The app will fetch news from the added feed automatically.
   
2. **Fetch and Display Updates**: 
   - After adding the feed, the app will keep fetching new articles and display them in real-time.

3. **Explore News Articles**: 
   - Browse through the list of articles available from the RSS feeds.
   - Tap on any article to read it in full.

4. **Bookmark Articles**: 
   - Save your favorite articles for later reading by bookmarking them.

5. **Offline Reading**: 
   - Save articles for offline reading when you don't have an internet connection.

6. **Share Articles**: 
   - You can share any article with others using the share feature.

Additional Features:
- **Custom Feed Support**: Add any RSS feed of your choice to get updates from multiple sources.
- **Search Functionality**: Search through articles by keywords to quickly find relevant information.
- **Night Mode**: Toggle between light and dark themes for comfortable reading in different environments.

For support, suggestions, or inquiries, feel free to contact us via email.
]]

    AlertDialog.Builder(activity)
        .setTitle("About RSS Reader")
        .setMessage(about_message)
        .setPositiveButton("OK", nil)
        .show()
end

about_button.onClick = show_about

function add_to_offline_news()
    Toast.makeText(activity, "Coming Soon...", Toast.LENGTH_SHORT).show()
end

require "import"
import "android.app.AlertDialog"
import "android.widget.Toast"
import "android.content.Intent"
import "android.net.Uri"

function read_offline_news()
    local message = [[
This feature is currently under development.

If you're an AndroLua developer, we’d love to hear from you!  
You can contribute by improving this code and sharing your version with us.

Note: This code is not encrypted. If you enhance it, feel free to send us your updated version via email.
]]

    AlertDialog.Builder(activity)
        .setTitle("Coming Soon")
        .setMessage(message)
        .setPositiveButton("Send Email", {
            onClick = function(dialog, which)
                -- Create an email intent
                local intent = Intent(Intent.ACTION_SENDTO)
                intent.setData(Uri.parse("mailto:chaudharisushil364@gmail.com")) -- Replace with your actual Gmail
                intent.putExtra(Intent.EXTRA_SUBJECT, "AndroLua Developer Contribution")
                intent.putExtra(Intent.EXTRA_TEXT, "Hello, I have made improvements to the code. Here is my version:")

                -- Start the email app
                if intent.resolveActivity(activity.getPackageManager()) then
                    activity.startActivity(intent)
                else
                    Toast.makeText(activity, "No email app found.", Toast.LENGTH_SHORT).show()
                end
            end
        })
        .setNegativeButton("Cancel", nil)
        .show()
end

offline_news_button.onClick = read_offline_news

-- Long press to show options for copying link, sharing, adding to offline, and adding to bookmarks
news_list.onItemLongClick = function(_, _, position, _)
    if news_data[position + 1] then
        local news_item = news_data[position + 1]
        local options = {"Copy Link", "Share", "Add to Offline", "Add to Bookmark"}

        -- Create a dialog with the options
        local builder = AlertDialog.Builder(activity)
        builder.setTitle("Choose an action")
        builder.setItems(options, { onClick = function(_, which)
            if which == 0 then  -- Copy Link
                copy_link(news_item.link)
            elseif which == 1 then  -- Share
                share_news(news_item)
            elseif which == 2 then  -- Add to Offline
                add_to_offline_news(news_item)
            elseif which == 3 then  -- Add to Bookmark
                add_to_bookmarks(news_item)
            end
        end })
        builder.show()
    end
    return true
end

function copy_link(item)
    -- Ensure valid input
    if not item or tostring(item) == "" then
        Toast.makeText(activity, "No valid link to copy", Toast.LENGTH_SHORT).show()
        return
    end

    local url = tostring(item)

    -- Get clipboard service
    local clipboard = activity.getSystemService(activity.CLIPBOARD_SERVICE)
    if not clipboard then
        Toast.makeText(activity, "Clipboard service unavailable", Toast.LENGTH_SHORT).show()
        return
    end

    -- Create a ClipData object
    local ClipData = luajava.bindClass("android.content.ClipData")
    local clip = ClipData.newPlainText("Copied Link", url)

    -- Copy to clipboard
    clipboard.setPrimaryClip(clip)

    Toast.makeText(activity, "Link copied to clipboard", Toast.LENGTH_SHORT).show()
end

-- Function to share the news item
function share_news(news_item)
    if not news_item.link or news_item.link == "" then
        Toast.makeText(activity, "No link to share", Toast.LENGTH_SHORT).show()
        return
    end
    local share_intent = Intent(Intent.ACTION_SEND)
    share_intent.setType("text/plain")
    share_intent.putExtra(Intent.EXTRA_TEXT, news_item.link)
    activity.startActivity(Intent.createChooser(share_intent, "Share News"))
    Toast.makeText(activity, "Sharing...", Toast.LENGTH_SHORT).show()
end